package au.id.mcauliffe.obscurer;

/* For license, see license.txt */

import java.util.Map;
import java.util.HashMap;

import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * A class that can use a variety of methods to obscure a string in a repeatable
 * way.  One example of this is using a hashing function.  The method used is 
 * dependant on the {@link ObscuringMethod} constant that is set for this
 * object.
 * <p>
 *     The purpose of this object is to create obscured versions of strings that
 *     may be used for equality comparisons.  This is useful for obscuring 
 *     passwords.  The obscured version is guaranteed to be the same for any 
 *     invocation with the same input.  It is not guaranteed, however, that the 
 *     output will be different for different inputs.  The degree of difference
 *     guarantee is dependant on the algorithm.  The output is in hex.
 * </p>
 */
public final class Obscurer
{
    private ObscuringMethod method = ObscuringMethod.MD5_HASH;
    private Map algorithms = new HashMap();
    
    /** Creates a new Obscurer, using the default obscuring method (MD5) */
    public Obscurer()
    {
    }
    
    /** Creates a new Obscurer, using the given obscuring method.
     *  @param method The method to use for obscuring strings
     */
    public Obscurer(ObscuringMethod method)
    {
        setMethod(method);
    }
    
    /** Sets the obscuring method to the given method.
     *  @param method the method to use for obscuring strings
     */
    public void setMethod(ObscuringMethod method)
    {
        this.method = method;
    }
    
    /** Returns the obscuring method currently set for this object.
     *  @return the obscuring method currently set for this object
     */
    public ObscuringMethod getMethod()
    {
        return method;
    }
    
    /** Obscures the given input using the current method.
     *  @param input the string to obscure
     *  @return the obscured version of the given string
     *  @throws ObscurerInitialisationException if the obscuring algorithm
     *                                          cannot be initialised
     *  @throws ObscurerException if the obscuring algorithm cannot be run
     */
    public String obscure(String input) throws ObscurerException
    {
        return getAlgorithm().obscure(input);
    }
    
    /** Obscures the given input using the given method.
     *  @param input the string to obscure
     *  @param method the method to use
     *  @return the obscured version of the given string
     *  @throws ObscurerInitialisationException if the obscuring algorithm
     *                                          cannot be initialised
     *  @throws ObscurerException if the obscuring algorithm cannot be run
     */
    public String obscure(String input, ObscuringMethod method) 
            throws ObscurerException
    {
        return getAlgorithm().obscure(input);
    }
    
    /** Run the obscurer on the standard input, outputing result to standard
     *  output. If a parameter is specified, it is the name of the method
     *  to use. (currently 'MD5' or 'SHA') Default is 'MD5'.
     *  <p>
     *      This object treats (obscures) each line of the input separately.
     *  </p>
     */
    public static void main(String[] args) throws Throwable
    {
        Obscurer obscurer;
        ObscuringMethod method = null;
        
        if (args.length > 0)
        {
            method = ObscuringMethod.getByName(args[0]);
        }
        
        if (method == null)
        {
            obscurer = new Obscurer();
        }
        else
        {
            obscurer = new Obscurer(method);
        }
        
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        
        String input = in.readLine();
        while (input != null)
        {
            System.out.println(obscurer.obscure(input));
            input = in.readLine();
        }
    }
    
    
    private ObscuringAlgorithm getAlgorithm()
    {
        return getAlgorithm(getMethod());
    }
    
    private ObscuringAlgorithm getAlgorithm(ObscuringMethod method)
    {
        ObscuringAlgorithm algorithm 
                = (ObscuringAlgorithm) algorithms.get(method);
        
        if (algorithm == null)
        {
            algorithm = getNewAlgorithm(method);
            algorithms.put(method,algorithm);
        }
        
        return algorithm;
    }
    
    private ObscuringAlgorithm getNewAlgorithm(ObscuringMethod method)
    {
        ObscuringAlgorithm algorithm = null;
        try
        {
              algorithm = (ObscuringAlgorithm) method.algorithmClass.newInstance();
        }
        catch(InstantiationException error)
        {
            throw new ObscurerInitialisationException(method,error);
        }
        catch(IllegalAccessException error)
        {
            throw new ObscurerInitialisationException(method,error);
        }
        
        return algorithm;
    }
}

